@@ -326,11 +326,13 @@ class AccountDialog(object):
 			self._smsNotificationButton.setEnabled(False)
 
 	@QtCore.pyqtSlot(int)
+	@misc_utils.log_exception(_moduleLogger)
 	def _on_notification_change(self, state):
 		self._update_notification_state()
 
 	@QtCore.pyqtSlot()
 	@QtCore.pyqtSlot(bool)
+	@misc_utils.log_exception(_moduleLogger)
 	def _on_clear(self, checked = False):
 		self._doClear = True
 		self._dialog.accept()
@@ -349,12 +351,14 @@ class SMSEntryWindow(object):
 	def __init__(self, parent, app, session, errorLog):
 		self._session = session
 		self._session.draft.recipientsChanged.connect(self._on_recipients_changed)
+
 		self._session.draft.sendingMessage.connect(self._on_op_started)
 		self._session.draft.calling.connect(self._on_op_started)
-		self._session.draft.cancelling.connect(self._on_op_started)
 		self._session.draft.calling.connect(self._on_calling_started)
-		self._session.draft.called.connect(self._on_op_finished)
+		self._session.draft.cancelling.connect(self._on_op_started)
+
 		self._session.draft.sentMessage.connect(self._on_op_finished)
+		self._session.draft.called.connect(self._on_op_finished)
 		self._session.draft.cancelled.connect(self._on_op_finished)
 		self._session.draft.error.connect(self._on_op_error)
 		self._errorLog = errorLog
@@ -387,6 +391,8 @@ class SMSEntryWindow(object):
 
 		self._characterCountLabel = QtGui.QLabel("0 (0)")
 		self._singleNumberSelector = QtGui.QComboBox()
+		self._singleNumbersCID = None
+		self._singleNumberSelector.activated.connect(self._on_single_change_number)
 		self._smsButton = QtGui.QPushButton("SMS")
 		self._smsButton.clicked.connect(self._on_sms_clicked)
 		self._smsButton.setEnabled(False)
@@ -434,7 +440,7 @@ class SMSEntryWindow(object):
 		self._scrollTimer.timeout.connect(self._on_delayed_scroll_to_bottom)
 
 		self._window.show()
-		self._update_recipients()
+		self._update_target_fields()
 
 	def close(self):
 		self._window.destroy()
@@ -448,6 +454,7 @@ class SMSEntryWindow(object):
 		self._characterCountLabel.setText("%d (%d)" % (numCharsLeftInText, numTexts))
 
 	def _update_button_state(self):
+		self._cancelButton.setEnabled(True)
 		if self._session.draft.get_num_contacts() == 0:
 			self._dialButton.setEnabled(False)
 			self._smsButton.setEnabled(False)
@@ -461,12 +468,17 @@ class SMSEntryWindow(object):
 				self._smsButton.setEnabled(True)
 		else:
 			self._dialButton.setEnabled(False)
-			self._smsButton.setEnabled(True)
+			if count == 0:
+				self._smsButton.setEnabled(False)
+			else:
+				self._smsButton.setEnabled(True)
 
-	def _update_recipients(self):
+	def _update_target_fields(self):
 		draftContactsCount = self._session.draft.get_num_contacts()
 		if draftContactsCount == 0:
+			self._clear_target_list()
 			self._window.hide()
+			self._singleNumbersCID = None
 		elif draftContactsCount == 1:
 			(cid, ) = self._session.draft.get_contacts()
 			title = self._session.draft.get_title(cid)
@@ -474,6 +486,7 @@ class SMSEntryWindow(object):
 			numbers = self._session.draft.get_numbers(cid)
 
 			self._targetList.setVisible(False)
+			self._clear_target_list()
 			if description:
 				self._history.setText(description)
 				self._history.setVisible(True)
@@ -481,6 +494,7 @@ class SMSEntryWindow(object):
 				self._history.setText("")
 				self._history.setVisible(False)
 			self._populate_number_selector(self._singleNumberSelector, cid, numbers)
+			self._singleNumbersCID = None
 
 			self._scroll_to_bottom()
 			self._window.setWindowTitle(title)
@@ -488,10 +502,7 @@ class SMSEntryWindow(object):
 			self._smsEntry.setFocus(QtCore.Qt.OtherFocusReason)
 		else:
 			self._targetList.setVisible(True)
-			while self._targetLayout.count():
-				removedLayoutItem = self._targetLayout.takeAt(self._targetLayout.count()-1)
-				removedWidget = removedLayoutItem.widget()
-				removedWidget.close()
+			self._clear_target_list()
 			for cid in self._session.draft.get_contacts():
 				title = self._session.draft.get_title(cid)
 				description = self._session.draft.get_description(cid)
@@ -518,12 +529,21 @@ class SMSEntryWindow(object):
 			self._history.setText("")
 			self._history.setVisible(False)
 			self._singleNumberSelector.setVisible(False)
+			self._singleNumbersCID = None
 
 			self._scroll_to_bottom()
 			self._window.setWindowTitle("Contacts")
 			self._window.show()
 			self._smsEntry.setFocus(QtCore.Qt.OtherFocusReason)
 
+	def _clear_target_list(self):
+		while self._targetLayout.count():
+			removedLayoutItem = self._targetLayout.takeAt(self._targetLayout.count()-1)
+			removedWidget = removedLayoutItem.widget()
+			removedWidget.hide()
+			removedWidget.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
+			removedWidget.close()
+
 	def _populate_number_selector(self, selector, cid, numbers):
 		selector.clear()
 
@@ -544,14 +564,16 @@ class SMSEntryWindow(object):
 			selector.setCurrentIndex(defaultIndex)
 		else:
 			selector.setEnabled(False)
-		callback = functools.partial(
-			self._on_change_number,
-			cid
-		)
-		callback.__name__ = "thanks partials for not having names and pyqt for requiring them"
-		selector.currentIndexChanged.connect(
-			QtCore.pyqtSlot(int)(callback)
-		)
+
+		if selector is not self._singleNumberSelector:
+			callback = functools.partial(
+				self._on_change_number,
+				cid
+			)
+			callback.__name__ = "thanks partials for not having names and pyqt for requiring them"
+			selector.activated.connect(
+				QtCore.pyqtSlot(int)(callback)
+			)
 
 	def _scroll_to_bottom(self):
 		self._scrollTimer.start()
@@ -564,12 +586,10 @@ class SMSEntryWindow(object):
 	def _on_sms_clicked(self, arg):
 		message = unicode(self._smsEntry.toPlainText())
 		self._session.draft.send(message)
-		self._smsEntry.setPlainText("")
 
 	@misc_utils.log_exception(_moduleLogger)
 	def _on_call_clicked(self, arg):
 		self._session.draft.call()
-		self._smsEntry.setPlainText("")
 
 	@QtCore.pyqtSlot()
 	@misc_utils.log_exception(_moduleLogger)
@@ -580,13 +600,28 @@ class SMSEntryWindow(object):
 	def _on_remove_contact(self, cid, toggled):
 		self._session.draft.remove_contact(cid)
 
+	@misc_utils.log_exception(_moduleLogger)
+	def _on_single_change_number(self, index):
+		# Exception thrown when the first item is removed
+		cid = self._singleNumbersCID
+		if cid is None:
+			_moduleLogger.error("Number change occurred on the single selector when in multi-selector mode (%r)" % index)
+			return
+		try:
+			numbers = self._session.draft.get_numbers(cid)
+		except KeyError:
+			_moduleLogger.error("Contact no longer available (or bizarre error): %r (%r)" % (cid, index))
+			return
+		number = numbers[index][0]
+		self._session.draft.set_selected_number(cid, number)
+
 	@misc_utils.log_exception(_moduleLogger)
 	def _on_change_number(self, cid, index):
 		# Exception thrown when the first item is removed
 		try:
 			numbers = self._session.draft.get_numbers(cid)
 		except KeyError:
-			_moduleLogger.error("Contact no longer available (or bizarre error): %r" % cid)
+			_moduleLogger.error("Contact no longer available (or bizarre error): %r (%r)" % (cid, index))
 			return
 		number = numbers[index][0]
 		self._session.draft.set_selected_number(cid, number)
@@ -594,7 +629,8 @@ class SMSEntryWindow(object):
 	@QtCore.pyqtSlot()
 	@misc_utils.log_exception(_moduleLogger)
 	def _on_recipients_changed(self):
-		self._update_recipients()
+		self._update_target_fields()
+		self._update_button_state()
 
 	@QtCore.pyqtSlot()
 	@misc_utils.log_exception(_moduleLogger)
@@ -612,6 +648,7 @@ class SMSEntryWindow(object):
 	@QtCore.pyqtSlot()
 	@misc_utils.log_exception(_moduleLogger)
 	def _on_op_finished(self):
+		self._smsEntry.setPlainText("")
 		self._smsEntry.setReadOnly(False)
 		self._cancelButton.setVisible(False)
 		self._smsButton.setVisible(True)
